#lang slideshow
(require slideshow/play
         "history.rkt"
         "contract+type.rkt"
         "in-file.rkt")

(provide cycle-slides)

(define (cycle-slides)
  (as-history
   #:edu 0

   (define title "Modules and Contracts and Types")
   
   (define mod0
     (in-file #:name "dummy"
              (blank 200 100)))
   (define ctc0 (contract-icon))
   (define typ0 (type-icon))

   (define mod (cc-superimpose mod0 (ghost ctc0) (ghost typ0)))
   (define ctc (cc-superimpose ctc0 (ghost mod)))
   (define typ (cc-superimpose typ0 (ghost mod)))
   
   (define top (ghost mod))
   (define bottom (ghost mod))
   
   (define implies (vc-append
                    (* 1 gap-size)
                    top
                    (colorize (scale (t "⇒") 2) "forestgreen")
                    bottom))

   (define-values (tx ty) (lt-find implies top))
   (define-values (bx by) (lt-find implies bottom))

   (define blx (- bx (* 3/4 (pict-width top))))
   (define brx (+ bx (* 3/4 (pict-width top))))
   (define bly (- by (* 1/2 (pict-height top))))
   (define bry bly)
   (define ty2 (- ty (* 1/5 (pict-height top))))
   
   (define (drift n from to curve)
     (+ (* n to)
        (* (- 1 n) from)
        (* (sin (* n pi)) curve)))

   (define (trio #:top top
                 #:bottom-then-top bottom-then-top
                 #:then-bottom then-bottom
                 #:skip-last? [skip-last? #f])
     (play-n
      #:title title
      #:skip-last? #t
      (lambda (n)
        (leftward
         (let ([n (fast-middle n)])
           (let* ([p implies]
                  [p (pin-over p
                               (- tx (* n 3/4 client-w)) ty
                               top)]
                  [p (pin-over p
                               (drift n bx tx 300)
                               (drift n by ty 0)
                               bottom-then-top)]
                  [p (pin-over p
                               (- bx (* (- 1 n) 3/4 client-w)) by
                               then-bottom)])
             p))))))

   (define (duo #:top-then-bottom top-then-bottom
                #:bottom-then-top bottom-then-top
                #:skip-last? [skip-last? #f])
     (play-n
      #:title title
      #:skip-last? #t
      (lambda (n)
        (leftward
         (let ([n (fast-middle n)])
           (let* ([p implies]
                  [p (pin-over p
                               (drift n tx bx -300)
                               (drift n ty by 0)
                               top-then-bottom)]
                  [p (pin-over p
                               (drift n bx tx 300)
                               (drift n by ty 0)
                               bottom-then-top)])
             p))))))

   (trio #:top mod
         #:bottom-then-top ctc
         #:then-bottom typ
         #:skip-last? #t)

   (trio #:top ctc
         #:bottom-then-top typ
         #:then-bottom mod
         #:skip-last? #t)

   (duo #:top-then-bottom typ
        #:bottom-then-top mod)

   (play-n
    #:title title
    #:skip-last? #t
    (lambda (n)
      (leftward
       (let ([n (fast-middle n)])
         (let* ([p implies]
                [p (pin-over p
                             tx
                             (drift n ty ty2 0)
                             mod)]
                [p (pin-over p
                             (drift n bx brx 0)
                             (drift n by bry 0)
                             typ)]
                [p (pin-over p
                             (drift n (* -1 client-w) blx 0)
                             (drift n by bly 0)
                             ctc)])
           p)))))

   (define (rot n p x y q)
     (define cw (/ (pict-width q) 2))
     (define ch (/ (pict-height q) 2))
     (define cx (+ x cw))
     (define cy (+ y ch))
     (define rx (/ (pict-width p) 2))
     (define ry (/ (pict-height p) 2))
     (define v (make-rectangular (- cx rx)
                                 (- ry cy)))
     (define v2 (make-polar (magnitude v)
                            (+ (angle v) (* n 2 pi))))
     (pin-over p
               (- (+ rx (real-part v2)) cw)
               (- ry (imag-part v2) ch)
               q))

   (define (spin n)
     (let* ([p implies]
            [p (rot n p tx ty2 mod)]
            [p (rot n p brx bry typ)]
            [p (rot n p blx bly ctc)])
       (leftward p)))

   (play-n
    #:title title
    #:skip-first? #t
    #:steps 25
    (lambda (n) (spin (* 1.5 (fast-middle n)))))

   (void)))

(module+ main
  (cycle-slides))

