#lang slideshow
(require slideshow/play
         "person.rkt"
         "paper.rkt"
         "book.rkt"
         "gear.rkt")

(provide res+edu-slides)

(define (classroom-slide #:paper? [paper? #t]
                         #:gear? [gear? #f]
                         #:slide [n (if gear? 1 0)]
                         #:paper-slide [paper-n 0]
                         #:gear-copy-n [gear-copy-n 0])
  (let* ([move-students '(1 2)]
         [ss (map (lambda (s) (scale s 0.75)) (make-students))]
         [g2s (map ghost ss)]
         [g1s (map ghost ss)]
         [g1 (group-students g1s)]
         [g2 (group-students (for/list ([g2 (in-list g2s)]
                                        [i (in-naturals)]
                                        #:when (memq i move-students))
                               g2))]
         [paper-icon ((if paper? values ghost) (paper #:badge textbook-title))]
         [gear-icon (and gear? (scale (gear) 0.5))]
         [paper-icon+gear (if gear?
                              (refocus (hc-append gap-size
                                                  gear-icon
                                                  paper-icon)
                                       paper-icon)
                              paper-icon)]
         [textbook-icon (textbook)]
         [g-gear-icon (and gear-icon (ghost (launder gear-icon)))]
         [textbook-icon+gear (if g-gear-icon
                                 (refocus (hc-append gap-size
                                                     textbook-icon
                                                     g-gear-icon)
                                          textbook-icon)
                                 textbook-icon)]
         [p (ht-append
             (* 6 gap-size)
             (vc-append gap-size
                        textbook-icon+gear
                        g1)
             (professor)
             (vc-append gap-size
                        paper-icon+gear
                        g2))]
         [p (for/fold ([p p]) ([s (in-list ss)]
                               [g1 (in-list g1s)]
                               [g2 (in-list g2s)]
                               [i (in-naturals)])
              (if (memq i move-students)
                  (slide-pict p s g1 g2 (fast-middle n))
                  (pin-over p
                            g1 lt-find
                            s)))]
         [p (if (gear-copy-n . > . 0)
                (let-values ([(sx sy) (lt-find p gear-icon)]
                             [(ex ey) (lt-find p g-gear-icon)])
                  (pin-over p
                            (+ (* ex gear-copy-n) (* sx (- 1 gear-copy-n)))
                            (+ (* ey gear-copy-n) (* sy (- 1 gear-copy-n))
                               (* -100 (sin (* gear-copy-n pi))))
                            gear-icon))
                p)]
         [p (if (paper-n . > . 0)
                (let-values ([(px py) (lt-find p paper-icon)]
                             [(tx ty) (lt-find p textbook-icon)])
                  (pin-under p
                             (+ (* tx paper-n) (* px (- 1 paper-n)))
                             (+ (* ty paper-n) (* py (- 1 paper-n))
                                (* -150 (sin (* paper-n pi))))
                             (rt-superimpose (scale paper-icon (- 1 (* 0.4 paper-n)))
                                             (ghost paper-icon))))
                p)])
    p))

(define (res+edu-slides)
  (define name "Research + Education")
  (slide #:name name (classroom-slide #:paper? #f))
  (play-n #:name name (lambda (n) (classroom-slide #:paper-slide (fast-middle n))) #:skip-last? #t)
  (play-n #:name name (lambda (n) (classroom-slide #:slide n)))
  (play-n #:name name (lambda (n) (classroom-slide #:gear? #t #:gear-copy-n (fast-middle n)))))

(module+ main
  (res+edu-slides))
